Module buffer

Module buffer 

Internal data structure used for buffering raw bytes

Structs

GrowableCircleBuf
A circular buffer that will automatically grow when empty to temporarily fit a payload greater than the capacity