Module http

Module http 

A non-blocking HTTP client

Structs

Http1RequestSerializer
A SerializeFrame impl for HTTP 1.x where SerializeFrame::SerializedFrame is an HttpRequest.
Http1ResponseDeserializer
A DeserializeFrame impl for HTTP 1.x where DeserializeFrame::DeserializedFrame is an http::Response.
HttpClient
A simple non-blocking HTTP 1.x client that does not attempt to reuse/keep-alive connections.
HttpClientSession
A Session created by the HttpClient.
PendingHttpResponse
A pending or active HttpRequest for a [PersistentHttpSession], which can be polled to completion.
PersistentHttpConnection
Encapsulates a HttpClientSession, enabling multiple requests to be queued and used on a single session.

Enums

HttpRequest
An HTTP Request, which can be represented by a hyperium_http::Request or a serialized HTTP payload.
Scheme

Traits

IntoBody
Extensible public trait to support serializing a variety of body types.