PersistentHttpConnection

Struct PersistentHttpConnection 

pub struct PersistentHttpConnection { /* private fields */ }
Encapsulates a HttpClientSession, enabling multiple requests to be queued and used on a single session.

It ensures only one request can be in flight at a time, and will buffer the request until the connection is established. This utilizes HTTP 1.x keep-alive headers to attempt to re-use the same connection for multiple requests.

Requests are started by calling PersistentHttpConnection::request, which will return a PendingHttpResponse. These pending http response will coordinate with one another to send requests and drive responses in the order the function was called.

An HTTP server may elect to close an http session at any time. When this occurs, a new PersistentHttpConnection must be created from the original HttpClient.

Implementations