Internal data structure used for buffering raw bytes
Structs
- Growable
Circle Buf - A circular buffer that will automatically grow when empty to temporarily fit a payload greater than the capacity
Internal data structure used for buffering raw bytes