pub struct QueueReceiver<Sess, Payload, QueueImpl>{ /* private fields */ }Expand description
Encapsulates a Session and a Queue, polling from the Queue to implement Receive.
This is meant to be coupled with a QueueCallback or QueueCallbackRef, transforming a push-oriented callback into a poll-oriented receiver.
Payloads dispatched to the QueueCallback/QueueCallbackRef are queued so that they may be polled via Receive::receive.
Queue is a trait, giving users have the flexibility to choose an implementation that satisfies their Publish/Sync requirements.
Implementations