A non-blocking HTTP client
Structs
- Http1
Request Serializer - A
SerializeFrameimpl for HTTP 1.x whereSerializeFrame::SerializedFrameis anHttpRequest. - Http1
Response Deserializer - A
DeserializeFrameimpl for HTTP 1.x whereDeserializeFrame::DeserializedFrameis anhttp::Response. - Http
Client - A simple non-blocking HTTP 1.x client that does not attempt to reuse/keep-alive connections.
- Http
Client Session - A
Sessioncreated by theHttpClient. - Pending
Http Response - A pending or active
HttpRequestfor a [PersistentHttpSession], which can be polled to completion. - Persistent
Http Connection - Encapsulates a
HttpClientSession, enabling multiple requests to be queued and used on a single session.
Enums
- Http
Request - An HTTP Request, which can be represented by a
hyperium_http::Requestor a serialized HTTP payload. - Scheme
Traits
- Into
Body - Extensible public trait to support serializing a variety of body types.